/**
 ****************************************************************************
 * @file    tmpm383_adc.h
 * @brief   This file provides all the functions prototypes for M383 ADC driver.
 * @version V2.0.2.1
 * @date    2015/05/21
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM383_ADC_H
#define __TMPM383_ADC_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM383.h"
#include "tx03_common.h"

/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @addtogroup ADC
  * @{
  */

/** @addtogroup ADC_Exported_types
  * @{
  */

#define IS_ADC_UNIT(param)          ((param) == TSB_AD)

/* macros to set Analog Input channel or Program number together with its Enable/Disable setting for the trigger registers */
#define TRG_ENABLE(AINx_ProgX)   ((uint8_t)(0x80U | (AINx_ProgX)))
#define TRG_DISABLE(AINx_ProgX)  ((uint8_t)(AINx_ProgX))

#define ADC_HOLD_FIX                ((uint32_t)0x09U << 3U)     /* must write 0x1001 to this area */
#define IS_ADC_SAMPLEHOLD(param)    ((param) == ADC_HOLD_FIX)

#define ADC_FC_DIVIDE_LEVEL_NONE    ((uint32_t)0x00U)
#define IS_ADC_PRESCALER(param)     ((param) == ADC_FC_DIVIDE_LEVEL_NONE)

    typedef enum {
        ADC_TRG_SW = 0U,
        ADC_TRG_CONSTANT = 1U,
        ADC_TRG_TIMER = 2U
    } ADC_TrgType;
#define IS_ADC_TRGTYPE(param)       ((param) <= ADC_TRG_TIMER)
#define IS_ADC_START(param)         ((param) <= ADC_TRG_CONSTANT)

    typedef enum {
        ADC_10BITS = 0U,
        ADC_12BITS = 1U,
        ADC_RESERVEDBITS = 2U
    } ADC_Resolution;
#define IS_ADC_RESOLUTION(param)  ((param) <= ADC_12BITS)

    typedef enum {
        ADC_CMPCR_0 = 0U,
        ADC_CMPCR_1 = 1U
    } ADC_CMPCRx;
#define IS_ADC_CMPCRx(param)        ((param) <= ADC_CMPCR_1)

    typedef enum {
        ADC_REG0 = 0U,
        ADC_REG1 = 1U,
        ADC_REG2 = 2U,
        ADC_REG3 = 3U,
        ADC_REG4 = 4U,
        ADC_REG5 = 5U,
        ADC_REG6 = 6U,
        ADC_REG7 = 7U,
        ADC_REG8 = 8U,
        ADC_REG9 = 9U,
        ADC_REG10 = 10U,
        ADC_REG11 = 11U
    } ADC_REGx;
#define IS_ADC_REGx(param)          ((param) <= ADC_REG11)

    typedef enum {
        ADC_LARGER_THAN_CMP_REG = 0U,
        ADC_SMALLER_THAN_CMP_REG = 1U
    } ADC_CmpCondition;
#define IS_ADC_CMPCONDITION(param)   ((param) <= ADC_SMALLER_THAN_CMP_REG)

/**
  * @brief  ADC Monitor Init Structure definition
  */
    typedef struct {

        ADC_CMPCRx CMPCRx;      /*!< Select ADC_CMPCR_0 for ADxCMPCR0 or ADC_CMPCR_1 for ADxCMPCR1  x = B */
        ADC_REGx ResultREGx;    /*!< Select which result register among ADC_REG0 to ADC_REG11 will be used */
        uint32_t CmpTimes;      /*!< How many times will Comparison be counted, range from 1 to 16 */
        ADC_CmpCondition Condition;     /*!< Condition to compare ADxREGm with ADxCMPn. (x = B; m = 0 to 11; n = 0, 1) */
        uint32_t CmpValue;      /*!< Comparison value to be set in ADxCMP0 or ADxCMP1, max value is 4095 . ( x = B ) */
    } ADC_MonitorTypeDef;
#define IS_ADC_CMPTIMES(param)  (((param) >= 1U) && ((param) <= 16U))
#define IS_ADC_CMPVALUE_12BIT(param)   ((param) <= 4095U)
#define IS_ADC_CMPVALUE_10BIT(param)   ((param) <= 1023U)

    typedef enum {
        ADC_AIN0 = 0U,          /*!< define for Analog Input channel */
        ADC_AIN1 = 1U,
        ADC_AIN2 = 2U,
        ADC_AIN3 = 3U,
        ADC_AIN4 = 4U,
        ADC_AIN5 = 5U,
        ADC_AIN6 = 6U,
        ADC_AIN7 = 7U,
        ADC_AIN8 = 8U,
        ADC_AIN9 = 9U
    } ADC_AINx;

/* Analog Input channel to be set for trigger register, use with macro TRG_ENABLE() and TRG_DISABLE() */

#define IS_ADC_AINx(param)   (( ((param) >= TRG_ENABLE(ADC_AIN0))  && \
                                ((param) <= TRG_ENABLE(ADC_AIN9)) ) || \
                                ((param) <= TRG_DISABLE(ADC_AIN9)) )

/**
  * @brief Union to store ADC result
  */
    typedef union {
        uint32_t All;
        struct {
            uint32_t Stored:1;  /*!< bit0, AD result has been stored */
            uint32_t OverRun:1; /*!< bit1, OverRun, new AD result is stored before the old one been read */
            uint32_t Reserved1:2;       /*!< bit2 to bit3, reserved */
            uint32_t ADResult:12;       /*!< bit4 to bit15, store AD result */
            uint32_t Reserved2:16;      /*!< bit16 to bit31, reserved */
        } Bit;
    } ADC_Result;

/** @} */
/* End of group ADC_Exported_types */

/** @defgroup ADC_Exported_FunctionPrototypes
  * @{
  */
    void ADC_SetClk(TSB_AD_TypeDef * ADx, uint32_t Sample_HoldTime, uint32_t Prescaler_Output);
    void ADC_SetResolution(TSB_AD_TypeDef * ADx, ADC_Resolution ADBits);
    ADC_Resolution ADC_GetResolution(TSB_AD_TypeDef * ADx);
    void ADC_Enable(TSB_AD_TypeDef * ADx);
    void ADC_Disable(TSB_AD_TypeDef * ADx);
    void ADC_Start(TSB_AD_TypeDef * ADx, ADC_TrgType Trg);
    void ADC_StopConstantTrg(TSB_AD_TypeDef * ADx);
    WorkState ADC_GetConvertState(TSB_AD_TypeDef * ADx, ADC_TrgType Trg);
    void ADC_SetLowPowerMode(TSB_AD_TypeDef * ADx, FunctionalState NewState);
    void ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_MonitorTypeDef * Monitor);
    void ADC_DisableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx);
    ADC_Result ADC_GetConvertResult(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx);
    void ADC_SetTimerTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx);
    void ADC_SetSWTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx);
    void ADC_SetConstantTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx);

/** @} */
/* End of group ADC_Exported_FunctionPrototypes */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX03_Periph_Driver */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /*__TMPM383_ADC_H */
