/**
 *******************************************************************************
 * @file    tmpm383_adc.c
 * @brief   This file provides API functions for ADC driver.
 * @version V2.0.2.1
 * @date    2015/05/22
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm383_adc.h"

#if defined(__TMPM383_ADC_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/** @defgroup ADC_Private_Defines
  * @{
  */

#define ADC_MASK_KEEP1BIT       (uint32_t)0x01U
#define ADC_MASK_KEEP2BITS      (uint32_t)0x03U
#define ADC_MASK_KEEP3BITS      (uint32_t)0x07U
#define ADC_MASK_KEEP4BITS      (uint32_t)0x0FU
#define ADC_MASK_KEEP5BITS      (uint32_t)0x1FU
#define ADC_MASK_KEEP6BITS      (uint32_t)0x3FU
#define ADC_MASK_KEEP7BITS      (uint32_t)0x7FU
#define ADC_MASK_KEEP8BITS      (uint32_t)0xFFU

/* NOTE: MOD3 bit7 to bit0 must be wrote: 0b 0110 0000 */
#define ADC_MOD3_FIX           (uint32_t)0x0060U

static void proc_xSET(volatile uint32_t * addr, ADC_REGx ResultREGx, uint8_t MacroAINx);

/** @} */
/* End of group ADC_Private_Defines */

/** @defgroup ADC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group ADC_Private_FunctionPrototypes */

/** @defgroup ADC_Private_Functions
  * @{
  */


static const volatile uint32_t *const ADC_REGx_Address[12U] = {
    &TSB_AD->REG0, &TSB_AD->REG1, &TSB_AD->REG2, &TSB_AD->REG3,
    &TSB_AD->REG4, &TSB_AD->REG5, &TSB_AD->REG6, &TSB_AD->REG7,
    &TSB_AD->REG8, &TSB_AD->REG9, &TSB_AD->REG10, &TSB_AD->REG11
};


/** @brief  To do the standard calculation for TSETxy, ASETxy, SSETxy (xy = 03, 47, 811)
  * such as: 
  *      tmp = ADx->TSETxy;
  *      tmp &= ~(0xFF << ((ResultREGx % 4U) * 8U));
  *      tmp |= (uint32_t) MacroAINx << ((ResultREGx % 4U) * 8U);
  *      ADx->TSETxy = tmp;
  *      
  * @param  addr: The address of TSETxy, ASETxy, SSETxy (xy = 03, 47, 811)
  * @param  ResultREGx: refer to same parameter of ADC_SetTimerTrg()
  * @param  MacroAINx: refer to same parameter of ADC_SetTimerTrg()
  * @retval None  
*/
static void proc_xSET(volatile uint32_t * addr, ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    uint32_t tmp = 0U;
    uint32_t shift_bytes = 0U;
    tmp = *addr;
    shift_bytes = ((uint32_t) ResultREGx % 4U) * 8U;    /* == 0, 8, 16, 24 */

    tmp &= (uint32_t) (~(0xFFUL << shift_bytes));       /* Clear bytes in uint32_t */
    tmp |= (uint32_t) MacroAINx << shift_bytes;

    *addr = tmp;
}

/** @} */
/* End of group ADC_Private_Functions */


/** @defgroup ADC_Exported_Functions
  * @{
  */

/**
  * @brief  Set ADC prescaler output(SCLK) of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  Sample_HoldTime: Select ADC sample hold time.
  *   This parameter can be one of the following values:
  *   ADC_HOLD_FIX
  * @param  Prescaler_Output: Select ADC prescaler output.
  *   This parameter can be one of the following values:
  *   ADC_FC_DIVIDE_LEVEL_NONE
  * @retval None
  */
void ADC_SetClk(TSB_AD_TypeDef * ADx, uint32_t Sample_HoldTime, uint32_t Prescaler_Output)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_SAMPLEHOLD(Sample_HoldTime));
    assert_param(IS_ADC_PRESCALER(Prescaler_Output));

    tmp = Sample_HoldTime | Prescaler_Output;
    ADx->CLK = tmp;
}

/**
  * @brief  Set ADC resolution. This function should be called from beginning. 
  *         When Power up reset, the ADC module is set in 12bits mode. 
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  ADBits: set AD resolution as 12bits or 10bits.
  *   This parameter can be one of the following values:
  *     ADC_10BITS, ADC_12BITS.  
  * @retval None
  */
void ADC_SetResolution(TSB_AD_TypeDef * ADx, ADC_Resolution ADBits)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_RESOLUTION(ADBits));

    tmp = ADx->MOD3 & (ADC_MASK_KEEP1BIT << 8U);     /* Only keep bit8: RCUT, */
    tmp |= (uint32_t) ADBits << 10U;    /* Then OR the data wanted */
    tmp |= ADC_MOD3_FIX;        /* NOTE: bit7 to bit0 must be wrote: 0b01 100 000 */
    ADx->MOD3 = tmp;
}

/**
  * @brief  Read AD resolution setting. 
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @retval Current AD resolution set, which can be:
  *     ADC_10BITS, ADC_12BITS, ADC_RESERVEDBITS.  
  */
ADC_Resolution ADC_GetResolution(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;
    ADC_Resolution ret = ADC_RESERVEDBITS;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    /* Read and mask to keep bit11 and bit10 then shift to lowest position */
    tmp = ADx->MOD3 & (ADC_MASK_KEEP2BITS << 10U);
    tmp >>= 10U;

    if (tmp == ADC_10BITS) {
        ret = ADC_10BITS;
    } else if (tmp == ADC_12BITS) {
        ret = ADC_12BITS;
    } else {
        ret = ADC_RESERVEDBITS;
    }

    return ret;
}

/**
  * @brief  Enable the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @retval None
  */
void ADC_Enable(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    /* Bit0 of MOD0 must be written '0' then set bit1(.DACON) */
    tmp |= (uint32_t) (0x01U << 1U);
    ADx->MOD0 = tmp;

    tmp = ADx->MOD1 & (ADC_MASK_KEEP1BIT << 0U);        /* Only keep bit0, then set bit7(.ADEN)  */
    tmp |= (uint32_t) (0x01U << 7U);
    ADx->MOD1 = tmp;
}

/**
  * @brief  Disable the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @retval None
  */
void ADC_Disable(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    /* Write '0' to bit1 and bit0  */
    ADx->MOD0 = tmp;

    tmp = ADx->MOD1 & (ADC_MASK_KEEP1BIT << 0U);        /* Only keep bit0  */
    ADx->MOD1 = tmp;
}

/**
  * @brief  Start the specified ADC unit with software trigger or constant trigger.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  Trg: Select trigger type.
  *   This parameter can be one of the following values:
  *   ADC_TRG_SW, ADC_TRG_CONSTANT
  * @retval None
  */
void ADC_Start(TSB_AD_TypeDef * ADx, ADC_TrgType Trg)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_START(Trg));

    if (Trg == ADC_TRG_SW) {
        tmp = ADx->MOD0 & (ADC_MASK_KEEP1BIT << 1U);    /* Only keep bit1, then set bit0(.ADSS) */
        tmp |= (uint32_t) (0x01U << 0U);
        ADx->MOD0 = tmp;
    } else {                    /* ADC_TRG_CONSTANT */
        tmp = ADx->MOD1 & (ADC_MASK_KEEP1BIT << 7U);    /* Only keep bit7, then set bit0(.ADAS) */
        tmp |= (uint32_t) (0x01U << 0U);
        ADx->MOD1 = tmp;
    }
}

/**
  * @brief  Stop the specified ADC unit when use constant trigger.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @retval None
  */
void ADC_StopConstantTrg(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    /* Clear ADxMOD1<ADAS> */
    tmp = ADx->MOD1 & (ADC_MASK_KEEP1BIT << 7U);        /* Only keep bit7 */
    ADx->MOD1 = tmp;
}

/**
  * @brief  Get the conversion state of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  Trg: Select trigger type.
  *   This parameter can be one of the following values:
  *   ADC_TRG_SW, ADC_TRG_CONSTANT, ADC_TRG_TIMER
  * @retval ADC conversion state.
  *   The value returned can be one of the followings:
  *   DONE, BUSY.
  */
WorkState ADC_GetConvertState(TSB_AD_TypeDef * ADx, ADC_TrgType Trg)
{
    WorkState ret = DONE;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_TRGTYPE(Trg));

    tmp = ADx->MOD2;
    if (Trg == ADC_TRG_SW) {
        tmp &= (ADC_MASK_KEEP1BIT << 1U);       /* Only keep bit1 */
    } else {
        tmp &= (ADC_MASK_KEEP1BIT << 0U);       /* Only keep bit0 */
    }

    if (tmp) {
        ret = BUSY;
    } else {
        /* Do nothing */
    }

    return ret;
}

/**
  * @brief  Set ADC module to run in Low Power mode or Normal mode.
  *         When Power up reset, the ADC module is set in Low Power Mode. 
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  NewState: to enable low power mode, or disable it to work in normal mode
  *   This parameter can be one of the following values:
  *     ENABLE , DISABLE.
  * @retval None 
  */
void ADC_SetLowPowerMode(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    tmp = ADx->MOD3 & (ADC_MASK_KEEP2BITS << 10U);   /* Only keep bit11 and bit10 */
    tmp |= ADC_MOD3_FIX;        /* NOTE: bit7 to bit0 must be wrote: 0b01 100 000 */

    if (NewState == ENABLE) {   /* If enable, Set bit8 */
        tmp |= (uint32_t) (0x01U << 8U);
    } else {
        /* Do nothing */
    }

    TSB_AD->MOD3 = tmp;
}

/**
  * @brief  Set the monitor function of the specified ADC unit and enable it.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  Monitor: The structure containing ADC Monitor configuration
  *         Refer to the members of ADC_MonitorTypeDef in .h file for more detail usage.
  * @retval None
  */
void ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_MonitorTypeDef * Monitor)
{
    uint32_t tmp = 0U;
    uint32_t tmpComp = 0U;
    ADC_Resolution res = ADC_10BITS;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_POINTER_NOT_NULL(Monitor));
    assert_param(IS_ADC_CMPCRx(Monitor->CMPCRx));
    assert_param(IS_ADC_REGx(Monitor->ResultREGx));
    assert_param(IS_ADC_CMPTIMES(Monitor->CmpTimes));
    assert_param(IS_ADC_CMPCONDITION(Monitor->Condition));

    res = ADC_GetResolution(ADx);
    if (res == ADC_10BITS) {
        assert_param(IS_ADC_CMPVALUE_10BIT(Monitor->CmpValue));
        tmpComp = (Monitor->CmpValue & 0x03FFU) << 6U;  /* 10bit mode: use bit15 to bit6 */
    } else {
        assert_param(IS_ADC_CMPVALUE_12BIT(Monitor->CmpValue));
        tmpComp = (Monitor->CmpValue & 0x0FFFU) << 4U;  /* 12bit mode: use bit15 to bit4 */
    }

    tmp |= (uint32_t) Monitor->ResultREGx;
    tmp |= (uint32_t) Monitor->Condition << 4U;
    tmp |= (uint32_t) (Monitor->CmpTimes - 1U) << 8U;

    tmp |= (uint32_t) (0x01U << 7U);    /* Set bit 7: ADxCMPCRn<CMPnEN> to enable Monitor function */

    if (Monitor->CMPCRx == ADC_CMPCR_0) {
        ADx->CMPCR0 = tmp;
        ADx->CMP0 = tmpComp;
    } else {
        ADx->CMPCR1 = tmp;
        ADx->CMP1 = tmpComp;
    }
}

/**
  * @brief  Disable the monitor function of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  CMPCRx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *   ADC_CMPCR_0, ADC_CMPCR_1
  * @retval None
  */
void ADC_DisableMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx CMPCRx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CMPCRx(CMPCRx));

    if (CMPCRx == ADC_CMPCR_0) {
        /* Read CMPCR0 then only keep bit4 to bit0, bit11 to bit8 */
        tmp = ADx->CMPCR0 & ((ADC_MASK_KEEP5BITS << 0U) | (ADC_MASK_KEEP4BITS << 8U));
        ADx->CMPCR0 = tmp;
    } else {
        /* Read CMPCR1 then only keep bit4 to bit0, bit11 to bit8 */
        tmp = ADx->CMPCR1 & ((ADC_MASK_KEEP5BITS << 0U) | (ADC_MASK_KEEP4BITS << 8U));
        ADx->CMPCR1 = tmp;
    }
}

/**
  * @brief  Get result from the specified AD Conversion Result Register.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADB
  * @param  ResultREGx: Select which ADC result register will be read.
  *   This parameter can be one of the following values:
  *   ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *   ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @retval A union with AD result and 2 bits extra information.
  *         Refer to the members of ADC_Result in .h file for more detail usage.
  */
ADC_Result ADC_GetConvertResult(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx)
{
    ADC_Result result = { 0U };
    ADC_Resolution res = ADC_10BITS;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));

    /* Read out result register wanted */
    /* Only keep bit15 to bit0 */
    result.All = (*(ADC_REGx_Address[ResultREGx])) & (uint32_t) 0xFFFFU;

    res = ADC_GetResolution(ADx);
    if (res == ADC_10BITS) {
        result.Bit.ADResult >>= 2U;     /* 10bit mode: use bit15 to bit6 */
    } else {
        /* Do nothing */                /* 12bit mode, use bit15 to bit4 */
    }

    return result;
}

/**
  * @brief  Set Timer Trigger Program Register of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC unit.
  *   This parameter can be one of the following values(12 registers):
  *   ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *   ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be input with macro as the format below:
  *   TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *   ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *   ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9
  * @retval None
  */
void ADC_SetTimerTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    if (ResultREGx >= ADC_REG8) {
        proc_xSET(&ADx->TSET811, ResultREGx, MacroAINx);
    } else if (ResultREGx >= ADC_REG4) {
        proc_xSET(&ADx->TSET47, ResultREGx, MacroAINx);
    } else {
        proc_xSET(&ADx->TSET03, ResultREGx, MacroAINx);
    }
}

/**
  * @brief  Set Software Trigger Program Register of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC unit.
  *   This parameter can be one of the following values(12 registers):
  *   ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *   ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be input with macro as the format below:
  *   TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *   ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *   ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9
  * @retval None
  */
void ADC_SetSWTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    if (ResultREGx >= ADC_REG8) {
        proc_xSET(&ADx->SSET811, ResultREGx, MacroAINx);
    } else if (ResultREGx >= ADC_REG4) {
        proc_xSET(&ADx->SSET47, ResultREGx, MacroAINx);
    } else {
        proc_xSET(&ADx->SSET03, ResultREGx, MacroAINx);
    }
}

/**
  * @brief  Set Constant Trigger Program Register of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_AD
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC unit.
  *   This parameter can be one of the following values(12 registers):
  *   ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *   ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be input with macro as the format below:
  *   TRG_ENABLE(x), TRG_DISABLE(x).
  *   'x' above can be one of the following values:
  *   ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4, ADC_AIN5, 
  *   ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9
  * @retval None
  */
void ADC_SetConstantTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));
    assert_param(IS_ADC_AINx(MacroAINx));

    if (ResultREGx >= ADC_REG8) {
        proc_xSET(&ADx->ASET811, ResultREGx, MacroAINx);
    } else if (ResultREGx >= ADC_REG4) {
        proc_xSET(&ADx->ASET47, ResultREGx, MacroAINx);
    } else {
        proc_xSET(&ADx->ASET03, ResultREGx, MacroAINx);
    }
}

/** @} */
/* End of group ADC_Exported_Functions */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM383_ADC_H) */
