/**
 *******************************************************************************
 * @file    tmpm383_cg.c
 * @brief   This file provides API functions for CG driver 
 * @version V2.0.2.1
 * @date    2015/06/06
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm383_cg.h"

#if defined(__TMPM383_CG_H)

/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup CG
  * @brief CG driver modules
  * @{
  */

/** @defgroup CG_Private_Defines
  * @{
  */

#define CG_FC_GEAR_MASK             ((uint32_t)0xFFFFFFF8)
#define CG_PRCK_MASK                ((uint32_t)0xFFFFF8FF)
#define CG_SCOUT_MASK               ((uint32_t)0xFFFCFFFF)
#define CG_WUODR_MASK               ((uint32_t)0x00073FF6)
#define CG_OSCCR_WRITE0_MASK        ((uint32_t)0xFFFFCF3F)
#define CG_OSCCR_WRITE1_MASK        ((uint32_t)0x00000030)

#define CG_OSCCR_WUEF_MASK          ((uint32_t)0x00000002)
#define CG_OSCCR_PLLON_MASK         ((uint32_t)0x00000004)
#define CG_OSCCR_OSCSEL_MASK        ((uint32_t)0x00020000)
#define CG_OSCCR_XTEN_MASK          ((uint32_t)0x00000200)
#define CG_STBYCR_RXEN_MASK         ((uint32_t)0x00000100)
#define CG_STBYCR_RXTEN_MASK        ((uint32_t)0x00000200)
#define CG_STBYCR_DRVE_MASK         ((uint32_t)0x00010000)
#define CG_PLLSEL_PLLSEL_MASK       ((uint32_t)0x00000001)
#define CG_CKSEL_SYSCKFLG_MASK      ((uint32_t)0x00000001)
#define CG_OSCCR_XEN1_MASK          ((uint32_t)0x00000100)
#define CG_OSCCR_XEN2_MASK          ((uint32_t)0x00010000)
#define CG_OSCCR_HOSCON_MASK        ((uint32_t)0x00040000)

#define CG_STBY_MODE_MASK           ((uint32_t)0xFFFFFFF8)
#define CG_NMIFLG_MASK              ((uint32_t)0xFFFFFFFE)
#define CG_RESET_FLAG_MASK          ((uint32_t)0xFFFFFFC8)

#define CG_FPSEL_MASK               ((uint32_t)0xFFFFCFFF)

#define FC_GEAR_1_1                 ((uint32_t)0x00000000)
#define FC_GEAR_1_2                 ((uint32_t)0x00000004)
#define FC_GEAR_1_4                 ((uint32_t)0x00000005)
#define FC_GEAR_1_8                 ((uint32_t)0x00000006)
#define FC_GEAR_1_16                ((uint32_t)0x00000007)

#define WARM_UP_SEL_OSC_EXT_HIGH    ((uint32_t)0x00080000)
#define WARM_UP_SEL_OSC_INT_HIGH    ((uint32_t)0x00000000)
#define WARM_UP_SEL_OSC_EXT_LOW     ((uint32_t)0x00000008)

#define SCOUT_SRC_FS                ((uint32_t)0x00000000)
#define SCOUT_SRC_HALF_FSYS         ((uint32_t)0x00010000)
#define SCOUT_SRC_FSYS              ((uint32_t)0x00020000)
#define SCOUT_SRC_PHIT0             ((uint32_t)0x00030000)

typedef union {
    uint32_t byte4;
    uint8_t byte[4];
} CG_Byte4;

static CG_DivideLevel numToDivideLevel_table[CG_DIVIDE_MAX] = {
    CG_DIVIDE_1,
    CG_DIVIDE_2,
    CG_DIVIDE_4,
    CG_DIVIDE_8,
    CG_DIVIDE_16,
    CG_DIVIDE_32,
    CG_DIVIDE_64,
    CG_DIVIDE_128,
    CG_DIVIDE_256,
    CG_DIVIDE_512,
    CG_DIVIDE_UNKNOWN,
};

static CG_PhiT0Src numPhiT0Src_table[CG_PHIT0_SRC_MAX] = {
    CG_PHIT0_SRC_FGEAR,
    CG_PHIT0_SRC_FC,
    CG_PHIT0_SRC_FSYS,
    CG_PHIT0_SRC_FSYS,
};

static CG_STBYMode numToSTBYMode_table[CG_STBY_MODE_MAX] = {
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_STOP,
    CG_STBY_MODE_SLEEP,
    CG_STBY_MODE_IDLE,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_UNKNOWN,
};

/** @} */
/* End of group CG_Private_Defines */

/** @defgroup CG_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group CG_Private_FunctionPrototypes */

/** @defgroup CG_Private_Functions
  * @{
  */

/** @} */
/* End of group CG_Private_Functions */

/** @defgroup CG_Exported_Functions
  * @{
  */

/**
  * @brief  Set dividing level between clock fgear and fc.
  * @param  DivideFgearFromFc: Dividing level between fgear and fc.
  *   This parameter can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4, CG_DIVIDE_8, CG_DIVIDE_16
  * @retval None
  */
void CG_SetFgearLevel(CG_DivideLevel DivideFgearFromFc)
{
    uint32_t gear = FC_GEAR_1_1;
    uint32_t regval = TSB_CG->SYSCR;

    /* Check the parameters */
    assert_param(IS_CG_FC_FGEAR_DIVIDE_LEVEL(DivideFgearFromFc));

    /* Set the value of fgear */
    switch (DivideFgearFromFc) {
    case CG_DIVIDE_1:
        gear = FC_GEAR_1_1;
        break;
    case CG_DIVIDE_2:
        gear = FC_GEAR_1_2;
        break;
    case CG_DIVIDE_4:
        gear = FC_GEAR_1_4;
        break;
    case CG_DIVIDE_8:
        gear = FC_GEAR_1_8;
        break;
    case CG_DIVIDE_16:
        gear = FC_GEAR_1_16;
        break;
    default:
        /* Do nothing */
        break;
    }

    regval &= CG_FC_GEAR_MASK;
    regval |= gear;
    TSB_CG->SYSCR = regval;
}

/**
  * @brief  Get dividing level between clock fgear and fc.
  * @param  None
  * @retval The dividing level between clock fgear and fc
  *   The value returned can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4,
  *   CG_DIVIDE_8, CG_DIVIDE_16 or CG_DIVIDE_UNKNOWN
  */
CG_DivideLevel CG_GetFgearLevel(void)
{
    CG_DivideLevel DivideFgearFromFc = CG_DIVIDE_UNKNOWN;
    uint32_t syscr = TSB_CG->SYSCR;
    syscr &= (~CG_FC_GEAR_MASK);

    switch (syscr) {
    case FC_GEAR_1_1:
        DivideFgearFromFc = CG_DIVIDE_1;
        break;
    case FC_GEAR_1_2:
        DivideFgearFromFc = CG_DIVIDE_2;
        break;
    case FC_GEAR_1_4:
        DivideFgearFromFc = CG_DIVIDE_4;
        break;
    case FC_GEAR_1_8:
        DivideFgearFromFc = CG_DIVIDE_8;
        break;
    case FC_GEAR_1_16:
        DivideFgearFromFc = CG_DIVIDE_16;
        break;
    default:
        /* Do nothing */
        break;
    }

    return DivideFgearFromFc;
}

/**
  * @brief  Set PhiT0 Source
  * @param  PhiT0Src: Select the PhiT0 Source
  *   This parameter can be one of the following values:
  *   CG_PHIT0_SRC_FGEAR, CG_PHIT0_SRC_FC or CG_PHIT0_SRC_FSYS
  * @retval None
  */
void CG_SetPhiT0Src(CG_PhiT0Src PhiT0Src)
{
    uint32_t regval = 0U;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_CG_PHIT0_SRC(PhiT0Src));

    regval = TSB_CG->SYSCR;
    regval &= CG_FPSEL_MASK;
    tmp = (uint32_t) PhiT0Src;
    regval |= (tmp << 12U);

    TSB_CG->SYSCR = regval;
}

/**
  * @brief  Get PhiT0 Source
  * @param  None
  * @retval The source of PhiT0
  *   The value returned can be one of the following values:
  *   CG_PHIT0_SRC_FGEAR, CG_PHIT0_SRC_FC or CG_PHIT0_SRC_FSYS
  */
CG_PhiT0Src CG_GetPhiT0Src(void)
{
    CG_PhiT0Src source = CG_PHIT0_SRC_FGEAR;
    uint32_t fpsel = (TSB_CG->SYSCR) & (~CG_FPSEL_MASK);

    source = numPhiT0Src_table[fpsel >> 12U];

    return source;
}

/**
  * @brief  Set dividing level between clock PhiT0 and fc/fsys.
  * @param  DividePhiT0FromFc: Dividing level between PhiT0 and fc/fsys.
  *   This parameter can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4, CG_DIVIDE_8,
  *   CG_DIVIDE_16, CG_DIVIDE_32, CG_DIVIDE_64, CG_DIVIDE_128,
  *   CG_DIVIDE_256 or CG_DIVIDE_512
  * @retval  Success or not
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetPhiT0Level(CG_DivideLevel DividePhiT0FromFc)
{
    uint32_t fprclk = 0U;
    CG_DivideLevel fgear = CG_DIVIDE_UNKNOWN;
    CG_PhiT0Src PhiT0_src = CG_PHIT0_SRC_FGEAR;
    Result retval = ERROR;
    uint32_t regval = TSB_CG->SYSCR;

    PhiT0_src = CG_GetPhiT0Src();

    if (PhiT0_src == CG_PHIT0_SRC_FSYS) {  /* phiT0 source is fsys, no dividing */
        /* Check the parameters */
        assert_param(IS_CG_FSYS_PHIT0_DIVIDE_LEVEL(DividePhiT0FromFc));

        /* No need to setting */
        retval = SUCCESS;
    } else if (PhiT0_src == CG_PHIT0_SRC_FC) {  /* phiT0 source is fc, max divider is 32 */
        /* Check the parameters */
        assert_param(IS_CG_FC_PHIT0_DIVIDE_LEVEL(DividePhiT0FromFc));

        fprclk = (uint32_t) DividePhiT0FromFc;
        regval &= CG_PRCK_MASK;
        fprclk <<= 8U;
        regval |= fprclk;
        TSB_CG->SYSCR = regval;
        retval = SUCCESS;
    } else {                                /* phiT0 source is fgear, max divider is 512 */
        /* Check the parameters */
        assert_param(IS_CG_FC_FGEAR_PHIT0_DIVIDE_LEVEL(DividePhiT0FromFc));

        fgear = CG_GetFgearLevel();
        if (fgear != CG_DIVIDE_UNKNOWN) {
            if ((DividePhiT0FromFc > fgear) || (DividePhiT0FromFc == fgear)) {
                fprclk = (uint32_t) DividePhiT0FromFc - (uint32_t) fgear;
                if (fprclk <= CG_DIVIDE_32) {
                    regval &= CG_PRCK_MASK;
                    fprclk <<= 8U;
                    regval |= fprclk;
                    TSB_CG->SYSCR = regval;
                    retval = SUCCESS;
                } else {
                    /* Do nothing */
                }
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get dividing level between clock phiT0 and fc/fsys.
  * @param  None
  * @retval The divide level between clock phiT0 and fc/fsys
  *   The value returned can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4, CG_DIVIDE_8,
  *   CG_DIVIDE_16, CG_DIVIDE_32, CG_DIVIDE_64, CG_DIVIDE_128,
  *   CG_DIVIDE_256, CG_DIVIDE_512 or CG_DIVIDE_UNKNOWN
  */
CG_DivideLevel CG_GetPhiT0Level(void)
{
    uint32_t fprclk = 0U;
    CG_DivideLevel fgear = CG_DIVIDE_UNKNOWN;
    CG_DivideLevel phiT0Level = CG_DIVIDE_UNKNOWN;
    CG_PhiT0Src PhiT0_src = CG_PHIT0_SRC_FGEAR;

    fprclk = TSB_CG->SYSCR & (~CG_PRCK_MASK);
    fprclk = (uint32_t) (fprclk >> 8U);
    PhiT0_src = CG_GetPhiT0Src();

    if (PhiT0_src == CG_PHIT0_SRC_FSYS) {
        phiT0Level = CG_DIVIDE_1;
    } else if (PhiT0_src == CG_PHIT0_SRC_FC) {
        phiT0Level = numToDivideLevel_table[fprclk];
    } else {
        fgear = CG_GetFgearLevel();
        if (fgear != CG_DIVIDE_UNKNOWN) {
            fprclk = fprclk + fgear;
            if (fprclk < CG_DIVIDE_UNKNOWN) {
                phiT0Level = numToDivideLevel_table[fprclk];
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }

    return phiT0Level;
}

/**
  * @brief Set the source of SCOUT output
  * @param Source: Select source of SCOUT
  *   This parameter can be one of the following values:
  *   CG_SCOUT_SRC_FS, CG_SCOUT_SRC_HALF_FSYS,
  *   CG_SCOUT_SRC_FSYS or CG_SCOUT_SRC_PHIT0
  * @retval None
  */
void CG_SetSCOUTSrc(CG_SCOUTSrc Source)
{
    /* Get the original SCOUT SRC value */
    uint32_t tmp = 0U;
    uint32_t regval = TSB_CG->SYSCR;

    /* Check the parameters */
    assert_param(IS_CG_SCOUT_SRC(Source));

    switch (Source) {
    case CG_SCOUT_SRC_FS:
        tmp = SCOUT_SRC_FS;
        break;
    case CG_SCOUT_SRC_HALF_FSYS:
        tmp = SCOUT_SRC_HALF_FSYS;
        break;
    case CG_SCOUT_SRC_FSYS:
        tmp = SCOUT_SRC_FSYS;
        break;
    case CG_SCOUT_SRC_PHIT0:
        tmp = SCOUT_SRC_PHIT0;
        break;
    default:
        /* Do nothing */
        break;
    }

    regval &= CG_SCOUT_MASK;
    regval |= tmp;
    TSB_CG->SYSCR = regval;
}

/**
  * @brief  Get the source of SCOUT output.
  * @param  None
  * @retval Source of SCOUT output
  *   The value returned can be one of the following values:
  *   CG_SCOUT_SRC_FS, CG_SCOUT_SRC_FSYS,
  *   CG_SCOUT_SRC_PHIT0 or CG_SCOUT_SRC_HALF_FSYS
  */
CG_SCOUTSrc CG_GetSCOUTSrc(void)
{
    uint32_t tmp = 0U;
    CG_SCOUTSrc source = CG_SCOUT_SRC_FS;
    tmp = TSB_CG->SYSCR & (~CG_SCOUT_MASK);

    switch (tmp) {
    case SCOUT_SRC_FS:
        source = CG_SCOUT_SRC_FS;
        break;
    case SCOUT_SRC_HALF_FSYS:
        source = CG_SCOUT_SRC_HALF_FSYS;
        break;
    case SCOUT_SRC_FSYS:
        source = CG_SCOUT_SRC_FSYS;
        break;
    case SCOUT_SRC_PHIT0:
        source = CG_SCOUT_SRC_PHIT0;
        break;
    default:
        /* Do nothing */
        break;
    }

    return source;
}

/**
  * @brief  Set the warm up time 
  * @param  Source: Select source of warm-up counter
  *   This parameter can be one of the following values:
  *   CG_WARM_UP_SRC_OSC_EXT_HIGH, CG_WARM_UP_SRC_OSC_INT_HIGH, CG_WARM_UP_SRC_OSC_EXT_LOW
  * @param  Time: Set count time for warm-up timer
  *   If Source is CG_WARM_UP_SRC_OSC_EXT_HIGH or CG_WARM_UP_SRC_OSC_INT_HIGH
  *   range of Time is 0x0 to 0xFFF
  *   If Source is CG_WARM_UP_SRC_OSC_EXT_LOW range of Time is 0x0 to 0x3FFF
  * @retval None
  */
void CG_SetWarmUpTime(CG_WarmUpSrc Source, uint16_t Time)
{
    uint32_t tmp = 0U;
    uint32_t regval = TSB_CG->OSCCR;

    /* Check the parameters */
    assert_param(IS_CG_WARM_UP_SRC(Source));

    regval &= CG_WUODR_MASK;

    switch (Source) {
    case CG_WARM_UP_SRC_OSC_EXT_HIGH:
        /* Check the parameters */
        assert_param(IS_CG_WARM_UP_TIME_HIGH_SPEED(Time));
        tmp = Time;
        tmp = tmp << 20U;
        regval |= tmp;
        regval |= WARM_UP_SEL_OSC_EXT_HIGH;
        break;
    case CG_WARM_UP_SRC_OSC_INT_HIGH:
        /* Check the parameters */
        assert_param(IS_CG_WARM_UP_TIME_HIGH_SPEED(Time));
        tmp = Time;
        tmp = tmp << 20U;
        regval |= tmp;
        regval |= WARM_UP_SEL_OSC_INT_HIGH;
        break;
    case CG_WARM_UP_SRC_OSC_EXT_LOW:
        /* Check the parameters */
        assert_param(IS_CG_WARM_UP_TIME_LOW_SPEED(Time));
        /* Get low 2 bits of warm-up time */
        tmp = Time;
        tmp = (tmp & 0x0003U) << 14U;
        regval |= tmp;
        /* Get high 12 bits of warm-up time */
        tmp = Time;
        tmp = (tmp & 0x3ffcU) << 18U;
        regval |= tmp;
        regval |= WARM_UP_SEL_OSC_EXT_LOW;
        break;
    default:
        /* Do nothing */
        break;
    }

    regval &= CG_OSCCR_WRITE0_MASK;     /* CGOSCCR bit 13, 12, 7, 6 write "0" */
    regval |= CG_OSCCR_WRITE1_MASK;     /* CGOSCCR bit 5, 4 write "1" */
    TSB_CG->OSCCR = regval;
}

/**
  * @brief  Start operation of warm up timer for oscillator.
  * @param  None
  * @retval None
  */
void CG_StartWarmUp(void)
{
    TSB_CG_OSCCR_WUEON = 1U;
}

/**
  * @brief Check whether warm up is completed or not.
  * @param  None
  * @retval The state of warm-up
  *   The value returned can be one of the following values:
  *   DONE or BUSY
  */
WorkState CG_GetWarmUpState(void)
{
    WorkState state = BUSY;
    uint32_t wuef = (TSB_CG->OSCCR) & CG_OSCCR_WUEF_MASK;

    if (wuef != CG_OSCCR_WUEF_MASK) {
        state = DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Enable PLL or disable it.
  * @param  NewState: New state of PLL
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval Success or not
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetPLL(FunctionalState NewState)
{
    Result retval = ERROR;
    CG_FcSrc fcsrc = CG_FC_SRC_FPLL;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_OSCCR_PLLON = 1U;
        retval = SUCCESS;
    } else {
        fcsrc = CG_GetFcSrc();
        if (fcsrc == CG_FC_SRC_FOSC) {  /* PLL is not used. So it can be shut off */
            /* Set register to disable PLL */
            TSB_CG_OSCCR_PLLON = 0U;
            retval = SUCCESS;
        } else {                /* fcsrc == CG_FC_SRC_FPLL. PLL is in use, so disabling it should be invalid */
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get the status of PLL operation
  * @param  None
  * @retval Run or stop
  *   The value returned can be one of the following values:
  *   ENABLE or DISABLE
  */
FunctionalState CG_GetPLLState(void)
{
    FunctionalState pllstate = DISABLE;
    uint32_t pllon = (TSB_CG->OSCCR) & CG_OSCCR_PLLON_MASK;

    if (pllon == CG_OSCCR_PLLON_MASK) {
        pllstate = ENABLE;
    } else {
        /* Do nothing */
    }

    return pllstate;
}

/**
  * @brief  Enable or disable high-speed oscillator (fosc)
  * @param  Source: Select clock source for fosc
  *   This parameter can be one of the following values:
  *   CG_FOSC_EHOSC or CG_FOSC_IHOSC
  * @param  NewState: oscillator is enabled or disabled
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval Success or failure
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetFosc(CG_FoscSrc Source, FunctionalState NewState)
{
    CG_FoscSrc fosc_src = CG_FOSC_IHOSC;
    CG_FsysSrc fsys_src = CG_FSYS_SRC_FGEAR;
    uint32_t hoscon = (TSB_CG->OSCCR) & CG_OSCCR_HOSCON_MASK;
    Result retval = ERROR;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_CG_FOSC_SRC(Source));

    switch (Source) {
    case CG_FOSC_EHOSC:
        if (hoscon == CG_OSCCR_HOSCON_MASK) {
            if (NewState == ENABLE) {
                /* Enable external high-speed oscillator (fEHOSC) */
                TSB_CG_OSCCR_XEN1 = 1U;
                retval = SUCCESS;
            } else {
                fosc_src = CG_GetFoscSrc();
                fsys_src = CG_GetFsysSrc();
                if ((fosc_src == CG_FOSC_EHOSC)
                    && (fsys_src == CG_FSYS_SRC_FGEAR)) {
                    /* fEHOSC is in use, so it can't be disabled */
                    retval = ERROR;
                } else {
                    /* Disable fEHOSC */
                    TSB_CG_OSCCR_XEN1 = 0U;
                    retval = SUCCESS;
                }
            }
        } else {
            /* Do nothing */
        }

        break;
    case CG_FOSC_IHOSC:
        if (NewState == ENABLE) {
            /* Enable internal high-speed oscillator (fIHOSC) */
            TSB_CG_OSCCR_XEN2 = 1U;
            retval = SUCCESS;
        } else {
            fosc_src = CG_GetFoscSrc();
            fsys_src = CG_GetFsysSrc();
            if ((fosc_src == CG_FOSC_IHOSC)
                && (fsys_src == CG_FSYS_SRC_FGEAR)) {
                /* fIHOSC is in use, so it can't be disabled */
                retval = ERROR;
            } else {
                /* Disable fIHOSC */
                TSB_CG_OSCCR_XEN2 = 0U;
                retval = SUCCESS;
            }
        }
        break;
    default:
        /* Do nothing */
        break;
    }

    return retval;
}

/**
  * @brief  Set the source of high-speed oscillator (fosc)
  * @param  Source: Select clock source for fosc
  *   This parameter can be one of the following values:
  *   CG_FOSC_EHOSC or CG_FOSC_IHOSC
  * @retval None
  */
void CG_SetFoscSrc(CG_FoscSrc Source)
{
    /* Check the parameters */
    assert_param(IS_CG_FOSC_SRC(Source));

    if (Source == CG_FOSC_EHOSC) {
        TSB_CG_OSCCR_OSCSEL = 1U;
    } else {
        TSB_CG_OSCCR_OSCSEL = 0U;
    }
}

/**
  * @brief  Get the source of high-speed oscillator (fosc)
  * @param  None
  * @retval Source of fosc
  *   The value returned can be one of the following values:
  *   CG_FOSC_EHOSC or CG_FOSC_IHOSC
  */
CG_FoscSrc CG_GetFoscSrc(void)
{
    CG_FoscSrc fosc_src = CG_FOSC_EHOSC;
    uint32_t oscsel = (TSB_CG->OSCCR) & CG_OSCCR_OSCSEL_MASK;

    if (oscsel != CG_OSCCR_OSCSEL_MASK) {
        fosc_src = CG_FOSC_IHOSC;
    } else {
        /* Do nothing */
    }

    return fosc_src;
}

/**
  * @brief  Get the state of high-speed oscillator (fosc)
  * @param  Source: Select fosc source
  *   This parameter can be one of the following values:
  *   CG_FOSC_EHOSC or CG_FOSC_IHOSC
  * @retval State of fosc
  *   The value returned can be one of the following values:
  *   ENABLE or DISABLE
  */
FunctionalState CG_GetFoscState(CG_FoscSrc Source)
{
    FunctionalState state = DISABLE;
    uint32_t xen1 = (TSB_CG->OSCCR) & CG_OSCCR_XEN1_MASK;
    uint32_t xen2 = (TSB_CG->OSCCR) & CG_OSCCR_XEN2_MASK;

    /* Check the parameters */
    assert_param(IS_CG_FOSC_SRC(Source));

    if (Source == CG_FOSC_EHOSC) {
        if (xen1 == CG_OSCCR_XEN1_MASK) {
            state = ENABLE;
        } else {
            /* Do nothing */
        }
    } else {    /* Source is IHOSC */
        if (xen2 == CG_OSCCR_XEN2_MASK) {
            state = ENABLE;
        } else {
            /* Do nothing */
        }
    }

    return state;
}

/**
  * @brief  Enable or disable low-speed oscillator (fs)
  * @param  NewState: oscillator is enabled or disabled
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval Success or failure
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetFs(FunctionalState NewState)
{
    CG_FsysSrc fsys_src = CG_FSYS_SRC_FGEAR;
    Result retval = ERROR;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Enable low-speed oscillator */
        TSB_CG_OSCCR_XTEN = 1U;
        retval = SUCCESS;
    } else {
        fsys_src = CG_GetFsysSrc();
        if (fsys_src == CG_FSYS_SRC_FS) {
            /* fs is in use, so it can't be disabled */
            retval = ERROR;
        } else {
            /* Disable low-speed oscillator */
            TSB_CG_OSCCR_XTEN = 0U;
            retval = SUCCESS;
        }
    }

    return retval;
}

/**
  * @brief  Get the state of low-speed oscillator (fs)
  * @param  None
  * @retval State of fs
  *   The value returned can be one of the following values:
  *   DISABLE or ENABLE
  */
FunctionalState CG_GetFsState(void)
{
    FunctionalState state = DISABLE;
    uint32_t xten = (TSB_CG->OSCCR) & CG_OSCCR_XTEN_MASK;

    if (xten == CG_OSCCR_XTEN_MASK) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Set port M as X1/X2 or general port
  * @param New mode of portM.
  *   This parameter can be one of the following values:
  *   CG_PORTM_AS_GPIO or CG_PORTM_AS_HOSC
  * @retval None
  */
void CG_SetPortM(CG_PortMMode Mode)
{
    /* Check the parameters */
    assert_param(IS_CG_PROTM_FUNCTION(Mode));

    TSB_CG_OSCCR_HOSCON = Mode;
}

/**
  * @brief  Set to the specified low-power mode
  * @param  Low power mode
  *   The value can be one of the following values:
  *   CG_STBY_MODE_STOP, CG_STBY_MODE_SLEEP, CG_STBY_MODE_IDLE
  * @retval None
  */
void CG_SetSTBYMode(CG_STBYMode Mode)
{
    uint32_t regval = TSB_CG->STBYCR;

    /* Check the parameter */
    assert_param(IS_CG_STBY_MODE(Mode));

    regval &= CG_STBY_MODE_MASK;
    regval |= (uint32_t) Mode;

    TSB_CG->STBYCR = regval;
}

/**
  * @brief  Get the low-power consumption mode
  * @param  None
  * @retval Low power mode
  *   The value returned can be one of the following values:
  *   CG_STBY_MODE_STOP, CG_STBY_MODE_SLEEP, CG_STBY_MODE_IDLE or CG_STBY_MODE_UNKNOWN
  */
CG_STBYMode CG_GetSTBYMode(void)
{
    CG_STBYMode mode = CG_STBY_MODE_UNKNOWN;
    uint32_t stby = (TSB_CG->STBYCR) & (~CG_STBY_MODE_MASK);

    mode = numToSTBYMode_table[stby];

    return mode;
}

/**
  * @brief  Enables or disables fosc after releasing stop mode
  * @param  NewState: Enable or disable
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval None
  */
void CG_SetExitStopModeFosc(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_STBYCR_RXEN = 1U;
    } else {
        TSB_CG_STBYCR_RXEN = 0U;
    }
}

/**
  * @brief  Get the state of fosc after releasing stop mode.
  * @param  None
  * @retval State of fosc after releasing stop mode
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  */
FunctionalState CG_GetExitStopModeFoscState(void)
{
    FunctionalState state = DISABLE;
    uint32_t rxen = (TSB_CG->STBYCR) & CG_STBYCR_RXEN_MASK;

    if (rxen == CG_STBYCR_RXEN_MASK) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Enable or disable fs after releasing stop mode
  * @param  NewState: enable or disable
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval None
  */
void CG_SetExitStopModeFs(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_STBYCR_RXTEN = 1U;
    } else {
        TSB_CG_STBYCR_RXTEN = 0U;
    }
}

/**
  * @brief  Get the state of fs after releasing stop mode.
  * @param  None
  * @retval State of fs after releasing stop mode
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  */
FunctionalState CG_GetExitStopModeFsState(void)
{
    FunctionalState state = DISABLE;
    uint32_t rxten = (TSB_CG->STBYCR) & CG_STBYCR_RXTEN_MASK;

    if (rxten == CG_STBYCR_RXTEN_MASK) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Set pin status in stop mode
  * @param  NewState: enable or disable
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE
  * @retval None
  */
void CG_SetPinStateInStopMode(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_STBYCR_DRVE = 1U;
    } else {
        TSB_CG_STBYCR_DRVE = 0U;
    }
}

/**
  * @brief  Get pin status in stop mode
  * @param  None
  * @retval State of pin in stop mode
  *   The value returned can be one of the following values:
  *   DISABLE or ENABLE
  */
FunctionalState CG_GetPinStateInStopMode(void)
{
    FunctionalState state = DISABLE;
    uint32_t drve = (TSB_CG->STBYCR) & CG_STBYCR_DRVE_MASK;

    if (drve == CG_STBYCR_DRVE_MASK) {
        /* Pin status in stop mode is active */
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Set the source of fc
  * @param  Source:  The source of fc
  *   This parameter can be one of the following values:
  *   CG_FC_SRC_FOSC or CG_FC_SRC_FPLL
  * @retval success or failure
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetFcSrc(CG_FcSrc Source)
{
    Result retval = ERROR;
    FunctionalState fosc_on = DISABLE;
    FunctionalState pllon = DISABLE;
    CG_FoscSrc fosc_src = CG_FOSC_EHOSC;

    /* Check the parameters */
    assert_param(IS_CG_FC_SRC(Source));

    if (Source == CG_FC_SRC_FOSC) {  /* fc source is fosc */
        fosc_src = CG_GetFoscSrc();
        if (fosc_src == CG_FOSC_EHOSC) {
            fosc_on = CG_GetFoscState(CG_FOSC_EHOSC);
            if (fosc_on == ENABLE) {
                TSB_CG_PLLSEL_PLLSEL = 0U;
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
        } else {
            fosc_on = CG_GetFoscState(CG_FOSC_IHOSC);
            if (fosc_on == ENABLE) {
                TSB_CG_PLLSEL_PLLSEL = 0U;
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
        }
    } else {                    /* fc source is fpll */
        pllon = CG_GetPLLState();
        if (pllon == ENABLE) {
            TSB_CG_PLLSEL_PLLSEL = 1U;
            retval = SUCCESS;
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get the source of fc
  * @param  None
  * @retval The source of fc
  *   The value returned can be one of the following values:
  *   CG_FC_SRC_FOSC or CG_FC_SRC_FPLL
  */
CG_FcSrc CG_GetFcSrc(void)
{
    CG_FcSrc fcsrc = CG_FC_SRC_FOSC;
    uint32_t pllsel = (TSB_CG->PLLSEL) & CG_PLLSEL_PLLSEL_MASK;

    if (pllsel == CG_PLLSEL_PLLSEL_MASK) {
        fcsrc = CG_FC_SRC_FPLL;
    } else {
        /* Do nothing */
    }

    return fcsrc;
}

/**
  * @brief  Set the source of fsys
  * @param  Source: Select the source of fsys
  *   This parameter can be one of the following values:
  *   CG_FSYS_SRC_FGEAR or CG_FSYS_SRC_FS
  * @retval Success or failure
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetFsysSrc(CG_FsysSrc Source)
{
    Result retval = ERROR;
    FunctionalState fosc_on = DISABLE;
    FunctionalState fs_on = DISABLE;
    CG_FoscSrc fosc_src = CG_FOSC_EHOSC;
    CG_FsysSrc fsyssrc = CG_FSYS_SRC_FGEAR;

    /* Check the parameters */
    assert_param(IS_CG_FSYS_SRC(Source));

    if (Source == CG_FSYS_SRC_FGEAR) {  /* fsys source is fgear */
        fosc_src = CG_GetFoscSrc();
        if (fosc_src == CG_FOSC_EHOSC) {
            fosc_on = CG_GetFoscState(CG_FOSC_EHOSC);
            if (fosc_on == ENABLE) {
                /* Set sysclk to fgear */
                TSB_CG_CKSEL_SYSCK = 0U;

                do {
                    fsyssrc = CG_GetFsysSrc();
                } while (fsyssrc != CG_FSYS_SRC_FGEAR);
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
        } else {
            fosc_on = CG_GetFoscState(CG_FOSC_IHOSC);
            if (fosc_on == ENABLE) {
                /* Set sysclk to fgear */
                TSB_CG_CKSEL_SYSCK = 0U;

                do {
                    fsyssrc = CG_GetFsysSrc();
                } while (fsyssrc != CG_FSYS_SRC_FGEAR);
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
        }
    } else {                           /* fsys source is fs */
        fs_on = CG_GetFsState();
        if (fs_on == ENABLE) {
            /* Set sysclk to fs */
            TSB_CG_CKSEL_SYSCK = 1U;
            do {
                fsyssrc = CG_GetFsysSrc();
            } while (fsyssrc != CG_FSYS_SRC_FS);
            retval = SUCCESS;
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get the clock source of fsys
  * @param  None
  * @retval Source of fsys
  *   The value returned can be one of the following values:
  *   CG_FSYS_SRC_FGEAR or CG_FSYS_SRC_FS
  */
CG_FsysSrc CG_GetFsysSrc(void)
{
    CG_FsysSrc fsyssrc = CG_FSYS_SRC_FGEAR;
    uint32_t sysckflg = (TSB_CG->CKSEL) & CG_CKSEL_SYSCKFLG_MASK;

    if (sysckflg == CG_CKSEL_SYSCKFLG_MASK) {
        fsyssrc = CG_FSYS_SRC_FS;
    } else {
        /* Do nothing */
    }

    return fsyssrc;
}

/**
  * @brief  Set the INT source for releasing low power mode.
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *   CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2, CG_INT_SRC_3,
  *   CG_INT_SRC_4, CG_INT_SRC_5, CG_INT_SRC_8, CG_INT_SRC_F,
  *   CG_INT_SRC_RTC, CG_INT_SRC_RMC_RX
  * @param  ActiveState: Select the active state for release trigger
  *   For CG_INT_SRC_RTC, this parameter can only be CG_INT_ACTIVE_STATE_FALLING
  *   For CG_INT_SRC_RMC_RX, this parameter can only be CG_INT_ACTIVE_STATE_RISING
  *   For the others interrupt source, this parameter can be one of the following values:
  *   CG_INT_ACTIVE_STATE_L, 
  *   CG_INT_ACTIVE_STATE_H, 
  *   CG_INT_ACTIVE_STATE_FALLING,
  *   CG_INT_ACTIVE_STATE_RISING,
  *   CG_INT_ACTIVE_STATE_BOTH_EDGES
  * @param  NewState: Enable or disable this release trigger
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval None
  */
void CG_SetSTBYReleaseINTSrc(CG_INTSrc INTSource,
                             CG_INTActiveState ActiveState, FunctionalState NewState)
{
    uint32_t num = 0U;
    uint8_t i = 0U;
    uint8_t j = 0U;
    CG_Byte4 regval = { 0U };

    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));
    assert_param(IS_CG_INT_ACTIVE_STATE(ActiveState));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (INTSource == CG_INT_SRC_RTC) {
        assert_param(IS_CG_INT_RTC_ACTIVE_STATE(ActiveState));
    } else if (INTSource == CG_INT_SRC_RMC_RX) {
        assert_param(IS_CG_INT_RMCRX_ACTIVE_STATE(ActiveState));
    } else {
        /* Do nothing */
    }

    num = (uint32_t) INTSource;
    i = (uint8_t) (num / 4U);
    j = (uint8_t) (num % 4U);

    switch (i) {
    case 0U:                   /* IMCGA */
        regval.byte4 = TSB_CG->IMCGA;
        regval.byte[j] &= 0x8EU;
        /* Setting the edge first */
        regval.byte[j] |= ActiveState;
        TSB_CG->IMCGA = regval.byte4;
        /* Enable interrupt source */
        regval.byte[j] |= NewState;
        TSB_CG->IMCGA = regval.byte4;
        break;

    case 1U:                   /* IMCGB */
        regval.byte4 = TSB_CG->IMCGB;
        regval.byte[j] &= 0x8EU;
        regval.byte[j] |= ActiveState;
        TSB_CG->IMCGB = regval.byte4;
        regval.byte[j] |= NewState;
        TSB_CG->IMCGB = regval.byte4;
        break;

    case 2U:                   /* IMCGC */
        regval.byte4 = TSB_CG->IMCGC;
        regval.byte[j] &= 0x8EU;
        regval.byte[j] |= ActiveState;
        TSB_CG->IMCGC = regval.byte4;
        regval.byte[j] |= NewState;
        TSB_CG->IMCGC = regval.byte4;
        break;

    case 3U:                   /* IMCGD */
        regval.byte4 = TSB_CG->IMCGD;
        regval.byte[j] &= 0x8EU;
        regval.byte[j] |= ActiveState;
        TSB_CG->IMCGD = regval.byte4;
        regval.byte[j] |= NewState;
        TSB_CG->IMCGD = regval.byte4;
        break;

    case 4U:                   /* IMCGE */
        regval.byte4 = TSB_CG->IMCGE;
        regval.byte[j] &= 0x8EU;
        regval.byte[j] |= ActiveState;
        TSB_CG->IMCGE = regval.byte4;
        regval.byte[j] |= NewState;
        TSB_CG->IMCGE = regval.byte4;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Get the active state of INT source standby clear request
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *   CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2, CG_INT_SRC_3,
  *   CG_INT_SRC_4, CG_INT_SRC_5, CG_INT_SRC_8, CG_INT_SRC_F,
  *   CG_INT_SRC_RTC, CG_INT_SRC_RMC_RX
  * @retval Active state of the input INT
  *   The value returned can be one of the following values:
  *   CG_INT_ACTIVE_STATE_FALLING, 
  *   CG_INT_ACTIVE_STATE_RISING,
  *   CG_INT_ACTIVE_STATE_BOTH_EDGES,
  *   CG_INT_ACTIVE_STATE_INVALID
  */
CG_INTActiveState CG_GetSTBYReleaseINTState(CG_INTSrc INTSource)
{
    CG_INTActiveState int_active_state = CG_INT_ACTIVE_STATE_INVALID;
    uint8_t i = 0U;
    uint8_t j = 0U;
    uint8_t tmp = 0U;
    uint32_t num = 0U;
    CG_Byte4 regval[5] = { {0U}, {0U}, {0U}, {0U}, {0U} };

    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));

    regval[0].byte4 = TSB_CG->IMCGA;
    regval[1].byte4 = TSB_CG->IMCGB;
    regval[2].byte4 = TSB_CG->IMCGC;
    regval[3].byte4 = TSB_CG->IMCGD;
    regval[4].byte4 = TSB_CG->IMCGE;

    num = (uint32_t) INTSource;
    i = (uint8_t) (num / 4U);
    j = (uint8_t) (num % 4U);
    tmp = regval[i].byte[j];
    tmp &= 0x0CU;

    switch (tmp) {
    case 0x04U:
        int_active_state = CG_INT_ACTIVE_STATE_RISING;
        break;
    case 0x08U:
        int_active_state = CG_INT_ACTIVE_STATE_FALLING;
        break;
    case 0x0CU:
        int_active_state = CG_INT_ACTIVE_STATE_BOTH_EDGES;
        break;
    default:
        /* Do nothing */
        break;
    }

    return (int_active_state);
}

/**
  * @brief  Clear the input INT request.
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *   CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2, CG_INT_SRC_3,
  *   CG_INT_SRC_4, CG_INT_SRC_5, CG_INT_SRC_8, CG_INT_SRC_F,
  *   CG_INT_SRC_RTC, CG_INT_SRC_RMC_RX
  * @retval None
  */
void CG_ClearINTReq(CG_INTSrc INTSource)
{
    uint32_t regval = INTSource;

    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));

    TSB_CG->ICRCG = regval;
}

/**
  * @brief  Get the NMI flag that shows who triggered NMI.
  * @param  None
  * @retval NMI flag
  */
CG_NMIFactor CG_GetNMIFlag(void)
{
    CG_NMIFactor cg_nmi_factor = { 0U };

    cg_nmi_factor.All = TSB_CG->NMIFLG & (~CG_NMIFLG_MASK);

    return cg_nmi_factor;
}

/**
  * @brief  Get the type of reset reason and clear the reset flag.
  * @param  None
  * @retval Reset flag
  */
CG_ResetFlag CG_GetResetFlag(void)
{
    CG_ResetFlag reset_flag = { 0U };

    reset_flag.All = TSB_CG->RSTFLG & (~CG_RESET_FLAG_MASK);
    TSB_CG->RSTFLG = 0U;

    return reset_flag;
}

/** @} */
/* End of group CG_Exported_Functions */

/** @} */
/* End of group CG */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM383_CG_H)  */
