/**
 *******************************************************************************
 * @file    tmpm383_dnf.c
 * @brief   This file provides API functions for DNF driver.
 * @version V2.0.2.1
 * @date    2015/06/02
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm383_dnf.h"

#if defined(__TMPM383_DNF_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup DNF 
  * @brief DNF driver modules
  * @{
  */
/** @defgroup DNF_Private_Defines
  * @{
  */

/** @} */
/* End of group DNF_Private_Defines */

/** @defgroup DNF_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group DNF_Private_FunctionPrototypes */

/** @defgroup DNF_Private_Functions
  * @{
  */

/** @} */
/* End of group group DNF_Private_Functions */

/** @defgroup DNF_Exported_Functions
  * @{
  */

/**
  * @brief  Select noise filter clock.
  * @param  FilterClk: Noise filter clock.
  *   This parameter can be one of the following values:
  *   DNF_FILTER_CLK_STOP, DNF_FILTER_CLK_FSYS_2, DNF_FILTER_CLK_FSYS_4,
  *   DNF_FILTER_CLK_FSYS_8, DNF_FILTER_CLK_FSYS_16, DNF_FILTER_CLK_FSYS_32,
  *   DNF_FILTER_CLK_FSYS_64 or DNF_FILTER_CLK_FSYS_128.
  * @retval None
  * @register The used register:
  *   NFCKCR<NFCKS[2:0]>
  */
void DNF_SetFilterClk(uint32_t FilterClk)
{
    /* Check the parameters */
    assert_param(IS_DNF_FILTER_CLK(FilterClk));

    TSB_NF->CKCR = FilterClk;
    while (TSB_NF->CKCR != FilterClk) {
      /* Wait for updating CKCR */
    }
}

/**
  * @brief  Get noise filter clock.
  * @param  None
  * @retval Noise filter clock
  *   The value returned can be one of the following values:
  *   DNF_FILTER_CLK_STOP, DNF_FILTER_CLK_FSYS_2, DNF_FILTER_CLK_FSYS_4,
  *   DNF_FILTER_CLK_FSYS_8, DNF_FILTER_CLK_FSYS_16, DNF_FILTER_CLK_FSYS_32,
  *   DNF_FILTER_CLK_FSYS_64 or DNF_FILTER_CLK_FSYS_128.
  * @register The used register:
  *   NFCKCR<NFCKS[2:0]>
  */
uint32_t DNF_GetFilterClk(void)
{
    uint32_t retval = TSB_NF->CKCR;

    return retval;
}

/**
  * @brief  Enable or disable the INT0 noise filter. 
  * @param  NewState: The INT0 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN0>
  */
void DNF_SetInt0Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN0 = 1U;
    } else {
        TSB_NF_ENCR_NFEN0 = 0U;
    }
}

/**
  * @brief  Get the INT0 noise filter state. 
  * @param  None
  * @retval The INT0 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN0>
  */
FunctionalState DNF_GetInt0FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN0 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT1 noise filter. 
  * @param  NewState: The INT1 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN1>
  */
void DNF_SetInt1Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN1 = 1U;
    } else {
        TSB_NF_ENCR_NFEN1 = 0U;
    }
}

/**
  * @brief  Get the INT1 noise filter state. 
  * @param  None
  * @retval The INT1 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN1>
  */
FunctionalState DNF_GetInt1FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN1 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT2 noise filter. 
  * @param  NewState: The INT2 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN2>
  */
void DNF_SetInt2Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN2 = 1U;
    } else {
        TSB_NF_ENCR_NFEN2 = 0U;
    }
}

/**
  * @brief  Get the INT2 noise filter state. 
  * @param  None
  * @retval The INT2 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN2>
  */
FunctionalState DNF_GetInt2FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN2 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT3 noise filter. 
  * @param  NewState: The INT3 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN3>
  */
void DNF_SetInt3Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN3 = 1U;
    } else {
        TSB_NF_ENCR_NFEN3 = 0U;
    }
}

/**
  * @brief  Get the INT3 noise filter state. 
  * @param  None
  * @retval The INT3 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN3>
  */
FunctionalState DNF_GetInt3FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN3 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT4 noise filter. 
  * @param  NewState: The INT4 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN4>
  */
void DNF_SetInt4Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN4 = 1U;
    } else {
        TSB_NF_ENCR_NFEN4 = 0U;
    }
}

/**
  * @brief  Get the INT4 noise filter state. 
  * @param  None
  * @retval The INT4 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN4>
  */
FunctionalState DNF_GetInt4FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN4 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT5 noise filter. 
  * @param  NewState: The INT5 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN5>
  */
void DNF_SetInt5Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN5 = 1U;
    } else {
        TSB_NF_ENCR_NFEN5 = 0U;
    }
}

/**
  * @brief  Get the INT5 noise filter state. 
  * @param  None
  * @retval The INT5 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN5>
  */
FunctionalState DNF_GetInt5FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN5 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INT8 noise filter. 
  * @param  NewState: The INT8 noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFEN8>
  */
void DNF_SetInt8Filter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFEN8 = 1U;
    } else {
        TSB_NF_ENCR_NFEN8 = 0U;
    }
}

/**
  * @brief  Get the INT8 noise filter state. 
  * @param  None
  * @retval The INT8 noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFEN8>
  */
FunctionalState DNF_GetInt8FilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFEN8 == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/**
  * @brief  Enable or disable the INTF noise filter. 
  * @param  NewState: The INTF noise filter state.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE.
  * @retval None
  * @register The used register:
  *   NFENCR<NFENF>
  */
void DNF_SetIntFFilter(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_NF_ENCR_NFENF = 1U;
    } else {
        TSB_NF_ENCR_NFENF = 0U;
    }
}

/**
  * @brief  Get the INTF noise filter state. 
  * @param  None
  * @retval The INTF noise filter state.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  * @register The used register:
  *   NFENCR<NFENF>
  */
FunctionalState DNF_GetIntFFilterState(void)
{
    FunctionalState retval = DISABLE;

    if (TSB_NF_ENCR_NFENF == 1U) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return retval;
}

/** @} */
/* End of group DNF_Exported_Functions */

/** @} */
/* End of group DNF */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM383_DNF_H) */
