/**
 *******************************************************************************
 * @file    tmpm383_fuart.c
 * @brief   This file provides API functions for Full UART driver.
 * @version V2.0.2.1
 * @date    2015/06/01
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm383_fuart.h"

#if defined(__TMPM383_FUART_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup FUART
  * @brief FUART driver modules
  * @{
  */

/** @defgroup FUART_Private_Defines
  * @{
  */
#define CR_UARTEN_SET                          ((uint32_t)0x00000001U)
#define CR_UARTEN_CLEAR                        ((uint32_t)0xFFFFFF7EU)
#define CR_WRITE0_BIT_MASK                     ((uint32_t)0xFFFFF379U)
#define PERIOD_DETECTION_CLEAR                 ((uint32_t)0xFFFFFF8FU)
#define TX_TERMINAL_MODE_CLEAR                 ((uint32_t)0xFFFFFFFDU)
#define TERMINAL_SELECTION_CLEAR               ((uint32_t)0xFFFFFFFBU)

#define RSR_ERROR_MASK          ((uint32_t)0x0000000F)
#define RSR_FERR_FLAG           ((uint32_t)0x00000001)
#define RSR_PERR_FLAG           ((uint32_t)0x00000002)
#define RSR_BERR_FLAG           ((uint32_t)0x00000004)
#define RSR_OERR_FLAG           ((uint32_t)0x00000008)

#define ECR_ERROR_CLEAR                        ((uint32_t)0x00000000U)

#define FR_BUSY_FLAG                           ((uint32_t)0x00000008U)

#define LCR_H_FIFO_EN_FLAG                     ((uint32_t)0x00000010U)
#define LCR_H_FIFO_EN_SET                      ((uint32_t)0x00000010U)
#define LCR_H_FIFO_EN_CLEAR                    ((uint32_t)0xFFFFFFEFU)
#define FR_TX_STORAGE_MASK                     ((uint32_t)0x000000A0U)
#define FR_RX_STORAGE_MASK                     ((uint32_t)0x00000050U)

#define FR_TX_STORAGE_NORMAL                   ((uint32_t)0x00000000U)
#define FR_TX_STORAGE_FULL                     ((uint32_t)0x00000020U)
#define FR_TX_STORAGE_EMPTY                    ((uint32_t)0x00000080U)
#define FR_TX_STORAGE_INVALID                  ((uint32_t)0x000000A0U)

#define FR_RX_STORAGE_NORMAL                   ((uint32_t)0x00000000U)
#define FR_RX_STORAGE_EMPTY                    ((uint32_t)0x00000010U)
#define FR_RX_STORAGE_FULL                     ((uint32_t)0x00000040U)
#define FR_RX_STORAGE_INVALID                  ((uint32_t)0x00000050U)

#define LCR_H_WLEN_MASK                        ((uint32_t)0xFFFFFF9FU)
#define LCR_H_STP2_MASK                        ((uint32_t)0xFFFFFFF7U)
#define LCR_H_PARITY_MASK                      ((uint32_t)0xFFFFFF79U)
#define CR_FLOW_CTRL_MASK                      ((uint32_t)0x00000F00U)
#define CR_MODE_MASK                           ((uint32_t)0x0000CC00U)

#define LCR_H_BRK_SET                          ((uint32_t)0x00000001U)
#define LCR_H_BRK_CLEAR                        ((uint32_t)0xFFFFFFFEU)

#define FUART_INT_BITS                         ((uint32_t)0x000007F2U)

/** @} */
/* End of group FUART_Private_Defines */

/** @defgroup FUART_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group FUART_Private_FunctionPrototypes */

/** @defgroup FUART_Private_Functions
  * @{
  */

/** @} */
/* End of group FUART_Private_Functions */

/** @defgroup FUART_Exported_Functions
  * @{
  */

/**
  * @brief  Enable the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval None
  */
void FUART_Enable(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->CR | CR_UARTEN_SET;
    tmp &= CR_WRITE0_BIT_MASK;
    FUARTx->CR = tmp;
}

/**
  * @brief  Disable the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval None
  */
void FUART_Disable(TSB_UART_TypeDef * FUARTx)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    FUARTx->CR &= CR_UARTEN_CLEAR;
}

/**
  * @brief  Get received data from the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval The data received from FUARTx
  */
uint32_t FUART_GetRxData(TSB_UART_TypeDef * FUARTx)
{
    uint32_t retval = 0U;
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    /* Return received data */
    retval = FUARTx->DR & 0xFFU;

    return retval;
}

/**
  * @brief  Set data to be sent and start transmitting via the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  Data: The data to be sent.
  *   The Data range is 0x00 to 0xFF.
  * @retval None
  */
void FUART_SetTxData(TSB_UART_TypeDef * FUARTx, uint32_t Data)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_FUART_DATA(Data));

    FUARTx->DR = Data & 0xFFU;
}

/**
  * @brief  Get receive error status only after reading data from UARTxDR.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval The error flag.
  *   The value returned can be one of the followings:
  *   FUART_NO_ERR, FUART_OVERRUN, FUART_PARITY_ERR, FUART_FRAMING_ERR 
      ,FUART_BREAK_ERR,FUART_ERRS.
  */
FUART_Err FUART_GetErrStatus(TSB_UART_TypeDef * FUARTx)
{
    FUART_Err retval = FUART_NO_ERR;
    uint32_t tmp = 0U;

    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = (FUARTx->RSR & RSR_ERROR_MASK);
    switch (tmp) {
    case RSR_FERR_FLAG:        /* Check overrun flag */
        retval = FUART_FRAMING_ERR;
        break;
    case RSR_PERR_FLAG:        /* Check parity flag */
        retval = FUART_PARITY_ERR;
        break;
    case RSR_BERR_FLAG:        /* Check break flag */
        retval = FUART_BREAK_ERR;
        break;
    case RSR_OERR_FLAG:        /* Check overrun flag */
        retval = FUART_OVERRUN;
        break;
    default:
        if (tmp != 0U) {
            /* more than one error */
            retval = FUART_ERRS;
        } else {
            /* Do nothing */
        }
        break;
    }
    return retval;
}

/**
  * @brief  Clear receive error status.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval None
  */
void FUART_ClearErrStatus(TSB_UART_TypeDef * FUARTx)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    /* Clear the receive error status */
    FUARTx->ECR &= ECR_ERROR_CLEAR;
}

/**
  * @brief  Get the state that whether the specified Full UART channel
  *         is transmitting data or stopped.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval The work state of specified Full UART channel
  *   it can be one of the following values:
  *   BUSY: The Full UART is transmitting data
  *   DONE: The Full UART has stopped transmitting data
  */
WorkState FUART_GetBusyState(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    WorkState retval = DONE;

    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->FR & FR_BUSY_FLAG;

    /* Check busy flag */
    if (tmp == FR_BUSY_FLAG) {
        retval = BUSY;
    } else {
        /* do nothing */
    }
    return retval;

}

/**
  * @brief  Get the FIFO or hold register status.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param Direction: The direction of Full UART.
  *   This parameter can be one of the following values:
  *   FUART_RX, FUART_TX
  * @retval The storage status.
  *   When FIFO is enabled, FUART_StorageStatus indicates the FIFO status.
  *   When FIFO is disabled, FUART_StorageStatus indicates the hold register status.
  */
FUART_StorageStatus FUART_GetStorageStatus(TSB_UART_TypeDef * FUARTx, FUART_Direction Direction)
{
    uint32_t fen = 0U;
    uint32_t src = 0U;
    FUART_StorageStatus retval = FUART_STORAGE_EMPTY;

    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_FUART_DIRECTION(Direction));

    fen = FUARTx->LCR_H & LCR_H_FIFO_EN_FLAG;
    src = FUARTx->FR;
    if (fen == LCR_H_FIFO_EN_FLAG) {    /* FIFO is enabled */
        if (Direction == FUART_TX) {    /* Get Transmit FIFO status */
            src = FUARTx->FR & FR_TX_STORAGE_MASK;
            switch (src) {
            case FR_TX_STORAGE_NORMAL:
                retval = FUART_STORAGE_NORMAL;
                break;
            case FR_TX_STORAGE_FULL:
                retval = FUART_STORAGE_FULL;
                break;
            case FR_TX_STORAGE_EMPTY:
                retval = FUART_STORAGE_EMPTY;
                break;
            case FR_TX_STORAGE_INVALID:
                retval = FUART_STORAGE_INVALID;
                break;
            default:
                /* do nothing */
                break;
            }
        } else {                /* Get Receive FIFO status */
            src = FUARTx->FR & FR_RX_STORAGE_MASK;
            switch (src) {
            case FR_RX_STORAGE_NORMAL:
                retval = FUART_STORAGE_NORMAL;
                break;
            case FR_RX_STORAGE_EMPTY:
                retval = FUART_STORAGE_EMPTY;
                break;
            case FR_RX_STORAGE_FULL:
                retval = FUART_STORAGE_FULL;
                break;
            case FR_RX_STORAGE_INVALID:
                retval = FUART_STORAGE_INVALID;
                break;
            default:
                /* do nothing */
                break;
            }
        }
    } else {                    /* FIFO is disabled */
        if (Direction == FUART_TX) {    /* Get Transmit hold register status */
            src = FUARTx->FR & FR_TX_STORAGE_MASK;
            switch (src) {
            case FR_TX_STORAGE_NORMAL:
            case FR_TX_STORAGE_INVALID:
                retval = FUART_STORAGE_INVALID;
                break;
            case FR_TX_STORAGE_FULL:
                retval = FUART_STORAGE_FULL;
                break;
            case FR_TX_STORAGE_EMPTY:
                retval = FUART_STORAGE_EMPTY;
                break;
            default:
                /* do nothing */
                break;
            }
        } else {                /* Get Receive hold register status */
            src = FUARTx->FR & FR_RX_STORAGE_MASK;
            switch (src) {
            case FR_RX_STORAGE_NORMAL:
            case FR_RX_STORAGE_INVALID:
                retval = FUART_STORAGE_INVALID;
                break;
            case FR_RX_STORAGE_EMPTY:
                retval = FUART_STORAGE_EMPTY;
                break;
            case FR_RX_STORAGE_FULL:
                retval = FUART_STORAGE_FULL;
                break;
            default:
                /* do nothing */
                break;
            }
        }
    }
    return retval;
}

/**
  * @brief  Initialize and configure the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  InitStruct: The structure containing basic Full UART configuration
  * @retval None
  */
void FUART_Init(TSB_UART_TypeDef * FUARTx, FUART_InitTypeDef * InitStruct)
{
    uint32_t tmp = 0U;
    uint32_t fuartclk = 0U;
    uint32_t ibd = 0U;          /* Interger part of baud rate divisor */
    uint32_t fbd = 0U;          /* Franctional part of baud rate divisor */
    uint32_t br = InitStruct->BaudRate; /* BaudRate */
    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_FUART_BAUDRATE(InitStruct->BaudRate));
    assert_param(IS_FUART_DATA_BITS(InitStruct->DataBits));
    assert_param(IS_FUART_STOPBITS(InitStruct->StopBits));
    assert_param(IS_FUART_PARITY(InitStruct->Parity));
    assert_param(IS_FUART_MODE(InitStruct->Mode));
    assert_param(IS_FUART_FLOW_CTRL(InitStruct->FlowCtrl));

    /* Get UARTCLK */
    SystemCoreClockUpdate();
    fuartclk = SystemCoreClock; /* UARTCLK = fsys */

    ibd = fuartclk / (16U * br);
    fbd = (8U * fuartclk + br - 128U * ibd * br) / (2U * br);

    if (fbd == 0U) {
        fbd = 1U;               /* Franctional part of baud rate divisor can not be 0x00 */
    } else {
        /* do nothing */
    }

    FUARTx->IBRD = ibd;         /* Set interger part of baud rate divisor */
    FUARTx->FBRD = fbd;         /* Set franctional part of baud rate divisor */

    tmp = FUARTx->LCR_H;

    tmp &= LCR_H_WLEN_MASK;
    tmp |= InitStruct->DataBits;

    tmp &= LCR_H_STP2_MASK;
    tmp |= InitStruct->StopBits;

    tmp &= LCR_H_PARITY_MASK;
    tmp |= InitStruct->Parity;

    FUARTx->LCR_H = tmp;        /* Set DataBits, StopBits, Parity */

    tmp = FUARTx->CR;

    tmp &= CR_FLOW_CTRL_MASK;
    tmp |= InitStruct->FlowCtrl;

    tmp &= CR_MODE_MASK;
    tmp |= InitStruct->Mode;

    FUARTx->CR = tmp;           /* Set Flow Control, Mode */
}

/**
  * @brief  Enable the transmit and receive FIFO.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval None
  */
void FUART_EnableFIFO(TSB_UART_TypeDef * FUARTx)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    FUARTx->LCR_H |= LCR_H_FIFO_EN_SET;
}

/**
  * @brief  Disable the transmit and receive FIFO and the mode will be changed.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval None
  */
void FUART_DisableFIFO(TSB_UART_TypeDef * FUARTx)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    FUARTx->LCR_H &= LCR_H_FIFO_EN_CLEAR;
}

/**
  * @brief  Generate the break condition for Full UART.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  NewState: New state of the FUART Send break.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void FUART_SetSendBreak(TSB_UART_TypeDef * FUARTx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set UARTxLCR_H<BRK> to  enable the send break to generate transmit break condition */
        FUARTx->LCR_H |= LCR_H_BRK_SET;
    } else {
        /* Clear UARTxLCR_H<BRK> to disable the send break */
        FUARTx->LCR_H &= LCR_H_BRK_CLEAR;
    }
}

/**
  * @brief  Set the Receive and Transmit interrupt FIFO level.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  RxLevel: Receive interrupt FIFO level.
  *   This parameter can be one of the following values:
  *   FUART_RX_FIFO_LEVEL_4:  The data in Receive FIFO become >= 4 words
  *   FUART_RX_FIFO_LEVEL_8:  The data in Receive FIFO become >= 8 words
  *   FUART_RX_FIFO_LEVEL_16: The data in Receive FIFO become >= 16 words
  *   FUART_RX_FIFO_LEVEL_24: The data in Receive FIFO become >= 24 words
  *   FUART_RX_FIFO_LEVEL_28: The data in Receive FIFO become >= 28 words
  * @param  TxLevel: Transmit interrupt FIFO level.
  *   This parameter can be one of the following values:
  *   FUART_TX_FIFO_LEVEL_4:  The data in Transmit FIFO become <= 4 words
  *   FUART_TX_FIFO_LEVEL_8:  The data in Transmit FIFO become <= 8 words
  *   FUART_TX_FIFO_LEVEL_16: The data in Transmit FIFO become <= 16 words
  *   FUART_TX_FIFO_LEVEL_24: The data in Transmit FIFO become <= 24 words
  *   FUART_TX_FIFO_LEVEL_28: The data in Transmit FIFO become <= 28 words
  * @retval None
  */
void FUART_SetINTFIFOLevel(TSB_UART_TypeDef * FUARTx, uint32_t RxLevel, uint32_t TxLevel)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_FUART_RX_FIFO_LEVEL(RxLevel));
    assert_param(IS_FUART_TX_FIFO_LEVEL(TxLevel));

    tmp = RxLevel | TxLevel;

    FUARTx->IFLS = tmp;
}

/**
  * @brief  Mask(Enable) interrupt source of the specified channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  IntMaskSrc: The interrupt source to be masked(enabled).
  * @retval None
  */
void FUART_SetINTMask(TSB_UART_TypeDef * FUARTx, uint32_t IntMaskSrc)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_INT_MASK_SRC(IntMaskSrc));

    FUARTx->IMSC = IntMaskSrc;
}

/**
  * @brief  Get the mask(enable) setting for each interrupt source.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval A data with union that it indicates the mask setting.
  */
FUART_INTStatus FUART_GetINTMask(TSB_UART_TypeDef * FUARTx)
{
    FUART_INTStatus retval = { 0U };
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    retval.All = FUARTx->IMSC & FUART_INT_BITS;
    return retval;
}

/**
  * @brief  Get the raw interrupt status of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval A data with union that indicates the raw interrupt status.
  */
FUART_INTStatus FUART_GetRawINTStatus(TSB_UART_TypeDef * FUARTx)
{
    FUART_INTStatus retval = { 0U };
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    retval.All = FUARTx->RIS & FUART_INT_BITS;
    return retval;
}

/**
  * @brief  Get the masked interrupt status of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @retval A data with union that indicates the masked interrupt status.
  */
FUART_INTStatus FUART_GetMaskedINTStatus(TSB_UART_TypeDef * FUARTx)
{
    FUART_INTStatus retval = { 0U };
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    retval.All = FUARTx->MIS & FUART_INT_BITS;
    return retval;
}

/**
  * @brief  Clear the interrupts of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  INTStatus: A data with union that indicates the interrupts to be cleared.
  * @retval None
  */
void FUART_ClearINT(TSB_UART_TypeDef * FUARTx, FUART_INTStatus INTStatus)
{
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_INT_MASK_SRC(INTStatus.All));

    FUARTx->ICR = INTStatus.All;
}

/**
  * @brief  Enable loop back test mode of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  
  * @retval None
  */
void FUART_EnableLoopBack(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->HCCR | LOOPBACK_EN_SET;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Disable loop back test mode of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  
  * @retval None
  */
void FUART_DisableLoopBack(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->HCCR & LOOPBACK_EN_CLEAR;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Enable 50% duty mode of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  
  * @retval None
  */
void FUART_EnableDuty(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->HCCR | DUTY_EN_SET;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Disable 50% duty mode of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  * @param  
  * @retval None
  */
void FUART_DisableDuty(TSB_UART_TypeDef * FUARTx)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));

    tmp = FUARTx->HCCR & DUTY_EN_CLEAR;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Set “0” period detection control of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  *         PeriodDetection: the width of “0” period detection.
  *   This parameter can be one of the following values:
  *   FUART_PERIOD_DETEC_1: 1/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_2: 2/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_3: 3/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_4: 4/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_5: 5/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_6: 6/16 width more than "0" detection
  *   FUART_PERIOD_DETEC_7: 7/16 width more than "0" detection
  * @param  
  * @retval None
  */
void FUART_SetPeriodDetection(TSB_UART_TypeDef * FUARTx, uint32_t PeriodDetection )
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_PERIOD_DETECTION(PeriodDetection));
    
    tmp = FUARTx->HCCR & PERIOD_DETECTION_CLEAR;
    tmp |= PeriodDetection;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Select transmission terminal mode of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  *         TxTerminalMode: Transmission terminal mode.
  *   This parameter can be one of the following values:
  *   FUART_1_TERMINAL: 1 terminal mode 
  *   FUART_2_TERMINAL: 2 terminal mode
  * @param  
  * @retval None
  */
void FUART_SetTxTerminalMode(TSB_UART_TypeDef * FUARTx, uint32_t TxTerminalMode)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_TX_TERMINAL_MODE(TxTerminalMode));
    
    tmp = FUARTx->HCCR & TX_TERMINAL_MODE_CLEAR;
    tmp |= TxTerminalMode;
    FUARTx->HCCR = tmp;
}

/**
  * @brief  Set Terminal selection to start the start bit of the specified Full UART channel.
  * @param  FUARTx: Select the Full UART channel.
  *   This parameter can be one of the following values:
  *   FUART0.
  *         StartBitTerminal: Terminal selection to start the start bit
  *   This parameter can be one of the following values:
  *   FUART_TXD50A: Select UTxTXD50A
  *   FUART_TXD50B: Select UTxTXD50B
  * @param  
  * @retval None
  */
void FUART_SetStartBitTerminal(TSB_UART_TypeDef * FUARTx, uint32_t StartBitTerminal)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_FUART_PERIPH(FUARTx));
    assert_param(IS_TERMINAL_SELECTION(StartBitTerminal));
    
    tmp = FUARTx->HCCR & TERMINAL_SELECTION_CLEAR;
    tmp |= StartBitTerminal;
    FUARTx->HCCR = tmp;
}

/** @} */
/* End of group FUART_Exported_Functions */

/** @} */
/* End of group FUART */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM383_FUART_H) */
